#ifndef __ieee1394_h__
#define __ieee1394_h__

#include "type.h"

#define tCode(id) /**/

struct Ieee1394
{
	struct PacketTransmit
	{
		RelativeField<U32,0x0, 4, 4> tCode;
		RelativeField<U32,0x0, 8, 2> rt;
		RelativeField<U32,0x0,10, 6> tLabel;
		RelativeField<U32,0x0,16, 3> speed;
		RelativeField<U32,0x0,23, 1> srcBusId;
	};
	struct PacketTransmitPhy: public PacketTransmit
	{
		RelativeRef<U32,0x4> quadlets; /* 2 */
		U32 buf[3];
	};

	struct PacketTransmitRequest:public PacketTransmit
	{
		RelativeField<U32,0x4, 0,16> dstOffsetHi;
		RelativeField<U32,0x4,16,16> dstBusId;
		RelativeField<U32,0x8, 0,32> dstOffsetLo;
	};

	struct PacketTransmitResponse:public PacketTransmit
	{
		RelativeField<U32,0x4,12, 4> rCode;
		RelativeField<U32,0x4,16,16> dstBusId;
	};

	struct PacketReceive
	{
		RelativeField<U32,0x0, 0, 4> reserve;
		RelativeField<U32,0x0, 4, 4> tCode;
		RelativeField<U32,0x0, 8, 2> rt;
		RelativeField<U32,0x0,10, 6> tLabel;
		RelativeField<U32,0x0,16,16> dstBusId;
	};

	struct PacketReceivePhy: public PacketReceive
	{
		tCode(0xe);
		RelativeRef<U32,0x4> quadlets; /* 2 */
		U32 buf[3];
	};

	struct PacketReceiveRequest:public PacketReceive
	{
		RelativeField<U32,0x4, 0,16> dstOffsetHi;
		RelativeField<U32,0x4,16,16> srcBusId;
		RelativeField<U32,0x8, 0,32> dstOffsetLo;
	};

	struct PacketReceiveResponse:public PacketReceive
	{
		RelativeField<U32,0x4,12, 4> rCode;
		RelativeField<U32,0x4,16,16> srcBusId;
	};

	struct PacketReceiveTail
	{
		RelativeField<U32,0x0, 0,16> timeStamp;
		RelativeField<U32,0x0,16,16> xferStatus;
	};

	struct Lock
	{
		RelativeField<U32,0xc, 0,16> extendedTCode;
	};

	struct NoData
	{
		U32 buf[3];
	};
	struct Quadlet
	{
		U32 buf[3];
		U32 data;
	};
	struct BlockNoData
	{
		RelativeField<U32,0xc,16,16> dataLength;
		U32 buf[4];
	};
	struct Block
	{
		RelativeField<U32,0xc,16,16> dataLength;
		U32 buf[4];
		U32 datas[0];
	};

	// on cause
	struct QuadletReadRequestTransmit :public PacketTransmitRequest,NoData
	{
	};
	struct QuadletReadResponseReceive :public PacketReceiveResponse,Quadlet
	{
		tCode(0x6);
	};
	struct QuadletWriteRequestTransmit :public PacketTransmitRequest,Quadlet
	{
	};
	struct QuadletWriteResponseReceive :public PacketReceiveResponse,NoData
	{
		tCode(0x2);
	};

	struct BlockReadRequestTransmit :public PacketTransmitRequest,BlockNoData
	{
	};
	struct BlockReadResponseReceive :public PacketReceiveResponse,Block
	{
		tCode(0x7);
	};
	struct BlockWriteRequestTransmit :public PacketTransmitRequest,Block
	{
	};
	struct BlockWriteResponseReceive :public PacketReceiveResponse,NoData
	{
		tCode(0x2);
	};
	struct LockRequestTransmit :public PacketTransmitRequest,Block,Lock
	{
	};
	struct LockResponseReceive :public PacketReceiveResponse,Block,Lock
	{
	};

	// on nous cause
	struct QuadletReadRequestReceive :public PacketReceiveRequest,NoData
	{
		tCode(0x4);
	};
	struct QuadletReadResponseTransmit :public PacketTransmitResponse,Quadlet
	{
	};
	struct QuadletWriteRequestReceive :public PacketReceiveRequest,Quadlet
	{
		tCode(0x0);
	};
	struct QuadletWriteResponseTransmit :public PacketTransmitResponse,NoData
	{
	};

	struct BlockReadRequestReceive :public PacketReceiveRequest,BlockNoData
	{
		tCode(0x5);
	};
	struct BlockReadResponseTransmit :public PacketTransmitResponse,Block
	{
	};
	struct BlockWriteRequestReceive :public PacketReceiveRequest,Block
	{
		tCode(0x1);
	};
	struct BlockWriteResponseTransmit :public PacketTransmitResponse,BlockNoData
	{
	};

	struct LockRequestReceive :public PacketReceiveRequest,Block,Lock
	{
		tCode(0x9);
	};
	struct LockResponseTransmit :public PacketTransmitResponse,Block,Lock
	{
	};
};


#endif /* __ieee1394_h__ */
